"""
Some constants related to specific Corruption items that the logic needs to deal with.
"""

ENERGY_CELL_MODEL = "Energy Cell"
ENERGY_CELL_ITEMS = [23 + i for i in range(9)]
ENERGY_CELL_TOTAL_ITEM = 22
